#!perl

BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

# This file was automatically generated by Dist::Zilla::Plugin::Author::GSG::HasVersionTests.
use strict;
use warnings;

use Test::More;

eval "use Test::HasVersion";
plan skip_all =>
     'Test::HasVersion required for testing for version numbers' if $@;

my $failed = 0;

for (all_pm_files()) {
    pm_version_ok($_) or $failed = 1;
}

diag(<<'EOL') if $failed;
To address these failed HasVersion tests, when using Author::GSG derived
Dist::Zilla PluginBundles, you should add a `# VERSION` comment to be used by
the OurPkgVersion Plugin.

See the documetation for Dist::Zilla::Plugin::Author::GSG::HasVersionTests
for further details.
EOL

done_testing
